      ! This is a comment line
      !
      PROGRAM  TestComment2
!        .........
         ! This is a comment line in the middle of a program
!        .........
      END PROGRAM  TestComment2


