<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<HTML><HEAD>
<TITLE>403 Forbidden</TITLE>
</HEAD><BODY>
<H1>Forbidden</H1>
You don't have permission to access /~shene/COURSES/cs201/NOTES/chap09/sums.f90
on this server.<P>
<HR>
<ADDRESS>Apache/1.3.22 Server at www.cs.mtu.edu Port 80</ADDRESS>
</BODY></HTML>
